@@ -29,7 +29,7 @@
 #include "glib-compat.h"
 
 typedef struct {
-  GTypelib *typelib;
+  GITypelib *typelib;
   GSList *context_stack;
 } ValidateContext;
 
@@ -56,7 +56,7 @@ validate_interface_blob (ValidateContext *ctx,
 			 GError       **error);
 
 static DirEntry *
-get_dir_entry_checked (GTypelib *typelib,
+get_dir_entry_checked (GITypelib *typelib,
 		       guint16    index,
 		       GError   **error)
 {
@@ -88,7 +88,7 @@ get_dir_entry_checked (GTypelib *typelib,
 
 
 static CommonBlob *
-get_blob (GTypelib *typelib,
+get_blob (GITypelib *typelib,
 	  guint32   offset,
 	  GError  **error)
 {
@@ -104,7 +104,7 @@ get_blob (GTypelib *typelib,
 }
 
 static InterfaceTypeBlob *
-get_type_blob (GTypelib *typelib,
+get_type_blob (GITypelib *typelib,
 	       SimpleTypeBlob *simple,
 	       GError  **error)
 {
@@ -131,7 +131,7 @@ get_type_blob (GTypelib *typelib,
 }
 
 DirEntry *
-g_typelib_get_dir_entry (GTypelib *typelib,
+g_typelib_get_dir_entry (GITypelib *typelib,
 			  guint16    index)
 {
   Header *header = (Header *)typelib->data;
@@ -139,6 +139,116 @@ g_typelib_get_dir_entry (GTypelib *typelib,
   return (DirEntry *)&typelib->data[header->directory + (index - 1) * header->entry_blob_size];
 }
 
+static Section *
+get_section_by_id (GITypelib   *typelib,
+		   SectionType  section_type)
+{
+  Header *header = (Header *)typelib->data;
+  Section *section;
+
+  if (header->sections == 0)
+    return NULL;
+
+  for (section = (Section*)&typelib->data[header->sections];
+       section->id != GI_SECTION_END;
+       section++)
+    {
+      if (section->id == section_type)
+	return section;
+    }
+  return NULL;
+}
+
+DirEntry *
+g_typelib_get_dir_entry_by_name (GITypelib *typelib,
+				 const char *name)
+{
+  Section *dirindex;
+  gint i;
+  const char *entry_name;
+  DirEntry *entry;
+
+  dirindex = get_section_by_id (typelib, GI_SECTION_DIRECTORY_INDEX);
+
+  if (dirindex == NULL)
+    {
+      gint n_entries = ((Header *)typelib->data)->n_local_entries;
+      for (i = 1; i <= n_entries; i++)
+	{
+	  entry = g_typelib_get_dir_entry (typelib, i);
+	  entry_name = g_typelib_get_string (typelib, entry->name);
+	  if (strcmp (name, entry_name) == 0)
+	    return entry;
+	}
+      return NULL;
+    }
+  else
+    {
+      guint8 *hash = (guint8*) &typelib->data[dirindex->offset];
+      guint16 index;
+
+      index = _gi_typelib_hash_search (hash, name);
+      entry = g_typelib_get_dir_entry (typelib, index + 1);
+      entry_name = g_typelib_get_string (typelib, entry->name);
+      if (strcmp (name, entry_name) == 0)
+	return entry;
+      return NULL;
+    }
+}
+
+DirEntry *
+g_typelib_get_dir_entry_by_gtype (GITypelib *typelib,
+				  gboolean   fastpass,
+				  GType      gtype)
+{
+  Header *header = (Header *)typelib->data;
+  guint n_entries = header->n_local_entries;
+  const char *gtype_name = g_type_name (gtype);
+  DirEntry *entry;
+  guint i;
+  const char *c_prefix;
+
+  /* Inside each typelib, we include the "C prefix" which acts as
+   * a namespace mechanism.  For GtkTreeView, the C prefix is Gtk.
+   * Given the assumption that GTypes for a library also use the
+   * C prefix, we know we can skip examining a typelib if our
+   * target type does not have this typelib's C prefix.
+   *
+   * However, not every class library necessarily conforms to this,
+   * e.g. Clutter has Cogl inside it.  So, we split this into two
+   * passes.  First we try a lookup, skipping things which don't
+   * have the prefix.  If that fails then we try a global lookup,
+   * ignoring the prefix.
+   *
+   * See http://bugzilla.gnome.org/show_bug.cgi?id=564016
+   */
+  c_prefix = g_typelib_get_string (typelib, header->c_prefix);
+  if (fastpass && c_prefix != NULL)
+    {
+      if (g_ascii_strncasecmp (c_prefix, gtype_name, strlen (c_prefix)) != 0)
+	return NULL;
+    }
+
+  for (i = 1; i <= n_entries; i++)
+    {
+      RegisteredTypeBlob *blob;
+      const char *type;
+
+      entry = g_typelib_get_dir_entry (typelib, i);
+      if (!BLOB_IS_REGISTERED_TYPE (entry))
+	continue;
+
+      blob = (RegisteredTypeBlob *)(&typelib->data[entry->offset]);
+      if (!blob->gtype_name)
+	continue;
+
+      type = g_typelib_get_string (typelib, blob->gtype_name);
+      if (strcmp (type, gtype_name) == 0)
+	return entry;
+    }
+  return NULL;
+}
+
 void
 g_typelib_check_sanity (void)
 {
@@ -203,7 +313,7 @@ is_aligned (guint32 offset)
 #define MAX_NAME_LEN 200
 
 static const char *
-get_string (GTypelib *typelib, guint32 offset, GError **error)
+get_string (GITypelib *typelib, guint32 offset, GError **error)
 {
   if (typelib->len < offset)
     {
@@ -218,7 +328,7 @@ get_string (GTypelib *typelib, guint32 offset, GError **error)
 }
 
 static const char *
-get_string_nofail (GTypelib *typelib, guint32 offset)
+get_string_nofail (GITypelib *typelib, guint32 offset)
 {
   const char *ret = get_string (typelib, offset, NULL);
   g_assert (ret);
@@ -226,7 +336,7 @@ get_string_nofail (GTypelib *typelib, guint32 offset)
 }
 
 static gboolean
-validate_name (GTypelib   *typelib,
+validate_name (GITypelib   *typelib,
 	       const char *msg,
 	       const guchar *data, guint32 offset,
 	       GError **error)
@@ -288,12 +398,12 @@ validate_header_basic (const guint8   *memory,
 
     }
 
-  if (header->major_version != 3 || header->minor_version != 0)
+  if (header->major_version != 4)
     {
       g_set_error (error,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID_HEADER,
-		   "Typelib version mismatch; expected 3, found %d",
+		   "Typelib version mismatch; expected 4, found %d",
 		   header->major_version);
       return FALSE;
 
@@ -386,7 +496,7 @@ static gboolean
 validate_header (ValidateContext  *ctx,
 		 GError          **error)
 {
-  GTypelib *typelib = ctx->typelib;
+  GITypelib *typelib = ctx->typelib;
   
   if (!validate_header_basic (typelib->data, typelib->len, error))
     return FALSE;
@@ -400,14 +510,14 @@ validate_header (ValidateContext  *ctx,
   return TRUE;
 }
 
-static gboolean validate_type_blob (GTypelib     *typelib,
+static gboolean validate_type_blob (GITypelib     *typelib,
 				    guint32        offset,
 				    guint32        signature_offset,
 				    gboolean       return_type,
 				    GError       **error);
 
 static gboolean
-validate_array_type_blob (GTypelib     *typelib,
+validate_array_type_blob (GITypelib     *typelib,
 			  guint32        offset,
 			  guint32        signature_offset,
 			  gboolean       return_type,
@@ -437,7 +547,7 @@ validate_array_type_blob (GTypelib     *typelib,
 }
 
 static gboolean
-validate_iface_type_blob (GTypelib     *typelib,
+validate_iface_type_blob (GITypelib     *typelib,
 			  guint32        offset,
 			  guint32        signature_offset,
 			  gboolean       return_type,
@@ -459,7 +569,7 @@ validate_iface_type_blob (GTypelib     *typelib,
 }
 
 static gboolean
-validate_param_type_blob (GTypelib     *typelib,
+validate_param_type_blob (GITypelib     *typelib,
 			  guint32        offset,
 			  guint32        signature_offset,
 			  gboolean       return_type,
@@ -502,7 +612,7 @@ validate_param_type_blob (GTypelib     *typelib,
 }
 
 static gboolean
-validate_error_type_blob (GTypelib     *typelib,
+validate_error_type_blob (GITypelib     *typelib,
 			  guint32        offset,
 			  guint32        signature_offset,
 			  gboolean       return_type,
@@ -554,7 +664,7 @@ validate_error_type_blob (GTypelib     *typelib,
 }
 
 static gboolean
-validate_type_blob (GTypelib     *typelib,
+validate_type_blob (GITypelib     *typelib,
 		    guint32        offset,
 		    guint32        signature_offset,
 		    gboolean       return_type,
@@ -568,16 +678,17 @@ validate_type_blob (GTypelib     *typelib,
   if (simple->flags.reserved == 0 &&
       simple->flags.reserved2 == 0)
     {
-      if (simple->flags.tag >= GI_TYPE_TAG_ARRAY)
+      if (!G_TYPE_TAG_IS_BASIC(simple->flags.tag))
 	{
 	  g_set_error (error,
 		       G_TYPELIB_ERROR,
 		       G_TYPELIB_ERROR_INVALID_BLOB,
-		       "Wrong tag in simple type");
+		       "Invalid non-basic tag %d in simple type", simple->flags.tag);
 	  return FALSE;
 	}
 
       if (simple->flags.tag >= GI_TYPE_TAG_UTF8 &&
+	  simple->flags.tag != GI_TYPE_TAG_UNICHAR &&
 	  !simple->flags.pointer)
 	{
 	  g_set_error (error,
@@ -632,7 +743,7 @@ validate_type_blob (GTypelib     *typelib,
 }
 
 static gboolean
-validate_arg_blob (GTypelib     *typelib,
+validate_arg_blob (GITypelib     *typelib,
 		   guint32        offset,
 		   guint32        signature_offset,
 		   GError       **error)
@@ -662,7 +773,7 @@ validate_arg_blob (GTypelib     *typelib,
 }
 
 static SimpleTypeBlob *
-return_type_from_signature (GTypelib *typelib,
+return_type_from_signature (GITypelib *typelib,
 			    guint32   offset,
 			    GError  **error)
 {
@@ -690,7 +801,7 @@ return_type_from_signature (GTypelib *typelib,
 }
 
 static gboolean
-validate_signature_blob (GTypelib     *typelib,
+validate_signature_blob (GITypelib     *typelib,
 			 guint32        offset,
 			 GError       **error)
 {
@@ -737,7 +848,7 @@ validate_function_blob (ValidateContext *ctx,
 			guint16        container_type,
 			GError       **error)
 {
-  GTypelib *typelib = ctx->typelib;
+  GITypelib *typelib = ctx->typelib;
   FunctionBlob *blob;
   SignatureBlob *sigblob;
   gboolean is_method;
@@ -873,7 +984,7 @@ validate_callback_blob (ValidateContext *ctx,
 			guint32        offset,
 			GError       **error)
 {
-  GTypelib *typelib = ctx->typelib;
+  GITypelib *typelib = ctx->typelib;
   CallbackBlob *blob;
 
   if (typelib->len < offset + sizeof (CallbackBlob))
@@ -910,7 +1021,7 @@ validate_callback_blob (ValidateContext *ctx,
 }
 
 static gboolean
-validate_constant_blob (GTypelib     *typelib,
+validate_constant_blob (GITypelib     *typelib,
 			guint32        offset,
 			GError       **error)
 {
@@ -936,11 +1047,12 @@ validate_constant_blob (GTypelib     *typelib,
     0, /* GSLIST */
     0, /* GHASH */
     0, /* ERROR */
+    4 /* UNICHAR */
   };
   ConstantBlob *blob;
   SimpleTypeBlob *type;
 
-  g_assert (G_N_ELEMENTS (value_size) == GI_TYPE_TAG_ERROR + 1);
+  g_assert (G_N_ELEMENTS (value_size) == GI_TYPE_TAG_N_TYPES);
 
   if (typelib->len < offset + sizeof (ConstantBlob))
     {
@@ -1006,7 +1118,7 @@ validate_constant_blob (GTypelib     *typelib,
 }
 
 static gboolean
-validate_value_blob (GTypelib     *typelib,
+validate_value_blob (GITypelib     *typelib,
 		     guint32        offset,
 		     GError       **error)
 {
@@ -1034,7 +1146,7 @@ validate_field_blob (ValidateContext *ctx,
 		     guint32        offset,
 		     GError       **error)
 {
-  GTypelib *typelib = ctx->typelib;
+  GITypelib *typelib = ctx->typelib;
   Header *header = (Header *)typelib->data;
   FieldBlob *blob;
 
@@ -1066,7 +1178,7 @@ validate_field_blob (ValidateContext *ctx,
 }
 
 static gboolean
-validate_property_blob (GTypelib     *typelib,
+validate_property_blob (GITypelib     *typelib,
 			guint32        offset,
 			GError       **error)
 {
@@ -1095,7 +1207,7 @@ validate_property_blob (GTypelib     *typelib,
 }
 
 static gboolean
-validate_signal_blob (GTypelib     *typelib,
+validate_signal_blob (GITypelib     *typelib,
 		      guint32        offset,
 		      guint32        container_offset,
 		      GError       **error)
@@ -1164,7 +1276,7 @@ validate_signal_blob (GTypelib     *typelib,
 }
 
 static gboolean
-validate_vfunc_blob (GTypelib     *typelib,
+validate_vfunc_blob (GITypelib     *typelib,
 		     guint32        offset,
 		     guint32        container_offset,
 		     GError       **error)
@@ -1227,7 +1339,7 @@ validate_struct_blob (ValidateContext *ctx,
 		      guint16        blob_type,
 		      GError       **error)
 {
-  GTypelib *typelib = ctx->typelib;
+  GITypelib *typelib = ctx->typelib;
   StructBlob *blob;
   gint i;
   guint32 field_offset;
@@ -1324,7 +1436,7 @@ validate_enum_blob (ValidateContext *ctx,
 		    guint16        blob_type,
 		    GError       **error)
 {
-  GTypelib *typelib = ctx->typelib;
+  GITypelib *typelib = ctx->typelib;
   EnumBlob *blob;
   gint i;
 
@@ -1423,7 +1535,7 @@ validate_object_blob (ValidateContext *ctx,
 		      guint32        offset,
 		      GError       **error)
 {
-  GTypelib *typelib = ctx->typelib;
+  GITypelib *typelib = ctx->typelib;
   Header *header;
   ObjectBlob *blob;
   gint i;
@@ -1603,7 +1715,7 @@ validate_interface_blob (ValidateContext *ctx,
 			 guint32        offset,
 			 GError       **error)
 {
-  GTypelib *typelib = ctx->typelib;
+  GITypelib *typelib = ctx->typelib;
   Header *header;
   InterfaceBlob *blob;
   gint i;
@@ -1726,7 +1838,7 @@ validate_interface_blob (ValidateContext *ctx,
 }
 
 static gboolean
-validate_errordomain_blob (GTypelib     *typelib,
+validate_errordomain_blob (GITypelib     *typelib,
 			   guint32        offset,
 			   GError       **error)
 {
@@ -1734,7 +1846,7 @@ validate_errordomain_blob (GTypelib     *typelib,
 }
 
 static gboolean
-validate_union_blob (GTypelib     *typelib,
+validate_union_blob (GITypelib     *typelib,
 		     guint32        offset,
 		     GError       **error)
 {
@@ -1746,7 +1858,7 @@ validate_blob (ValidateContext *ctx,
 	       guint32          offset,
 	       GError         **error)
 {
-  GTypelib *typelib = ctx->typelib;
+  GITypelib *typelib = ctx->typelib;
   CommonBlob *common;
 
   if (typelib->len < offset + sizeof (CommonBlob))
@@ -1815,7 +1927,7 @@ static gboolean
 validate_directory (ValidateContext   *ctx,
 		    GError            **error)
 {
-  GTypelib *typelib = ctx->typelib;
+  GITypelib *typelib = ctx->typelib;
   Header *header = (Header *)typelib->data;
   DirEntry *entry;
   gint i;
@@ -1892,7 +2004,7 @@ static gboolean
 validate_attributes (ValidateContext *ctx,
 		     GError       **error)
 {
-  GTypelib *typelib = ctx->typelib;
+  GITypelib *typelib = ctx->typelib;
   Header *header = (Header *)typelib->data;
 
   if (header->size < header->attributes + header->n_attributes * sizeof (AttributeBlob))
@@ -1936,7 +2048,7 @@ prefix_with_context (GError **error,
 }
 
 gboolean
-g_typelib_validate (GTypelib     *typelib,
+g_typelib_validate (GITypelib     *typelib,
 		     GError       **error)
 {
   ValidateContext ctx;
@@ -1974,7 +2086,7 @@ g_typelib_error_quark (void)
 }
 
 static void
-_g_typelib_do_dlopen (GTypelib *typelib)
+_g_typelib_do_dlopen (GITypelib *typelib)
 {
   Header *header;
   const char *shlib_str;
@@ -2060,7 +2172,7 @@ _g_typelib_do_dlopen (GTypelib *typelib)
 }
 
 static inline void
-_g_typelib_ensure_open (GTypelib *typelib)
+_g_typelib_ensure_open (GITypelib *typelib)
 {
   if (typelib->open_attempted)
     return;
@@ -2069,28 +2181,28 @@ _g_typelib_ensure_open (GTypelib *typelib)
 }
 
 /**
- * g_typelib_new_from_memory:
+ * g_typelib_new_from_memory: (skip)
  * @memory: address of memory chunk containing the typelib
  * @len: length of memory chunk containing the typelib
  * @error: a #GError
  *
- * Creates a new #GTypelib from a memory location.  The memory block
+ * Creates a new #GITypelib from a memory location.  The memory block
  * pointed to by @typelib will be automatically g_free()d when the
  * repository is destroyed.
  *
- * Return value: the new #GTypelib
+ * Return value: the new #GITypelib
  **/
-GTypelib *
+GITypelib *
 g_typelib_new_from_memory (guint8  *memory, 
 			   gsize    len,
 			   GError **error)
 {
-  GTypelib *meta;
+  GITypelib *meta;
 
   if (!validate_header_basic (memory, len, error))
     return NULL;
 
-  meta = g_slice_new0 (GTypelib);
+  meta = g_slice_new0 (GITypelib);
   meta->data = memory;
   meta->len = len;
   meta->owns_memory = TRUE;
@@ -2100,26 +2212,26 @@ g_typelib_new_from_memory (guint8  *memory,
 }
 
 /**
- * g_typelib_new_from_const_memory:
+ * g_typelib_new_from_const_memory: (skip)
  * @memory: address of memory chunk containing the typelib
  * @len: length of memory chunk containing the typelib
  * @error: A #GError
  *
- * Creates a new #GTypelib from a memory location.
+ * Creates a new #GITypelib from a memory location.
  *
- * Return value: the new #GTypelib
+ * Return value: the new #GITypelib
  **/
-GTypelib *
+GITypelib *
 g_typelib_new_from_const_memory (const guchar *memory, 
 				 gsize         len,
 				 GError      **error)
 {
-  GTypelib *meta;
+  GITypelib *meta;
 
   if (!validate_header_basic (memory, len, error))
     return NULL;
 
-  meta = g_slice_new0 (GTypelib);
+  meta = g_slice_new0 (GITypelib);
   meta->data = (guchar *) memory;
   meta->len = len;
   meta->owns_memory = FALSE;
@@ -2129,26 +2241,26 @@ g_typelib_new_from_const_memory (const guchar *memory,
 }
 
 /**
- * g_typelib_new_from_mapped_file:
+ * g_typelib_new_from_mapped_file: (skip)
  * @mfile: a #GMappedFile, that will be free'd when the repository is destroyed
  * @error: a #GError
  *
- * Creates a new #GTypelib from a #GMappedFile.
+ * Creates a new #GITypelib from a #GMappedFile.
  *
- * Return value: the new #GTypelib
+ * Return value: the new #GITypelib
  **/
-GTypelib *
+GITypelib *
 g_typelib_new_from_mapped_file (GMappedFile  *mfile,
 				GError      **error)
 {
-  GTypelib *meta;
+  GITypelib *meta;
   guint8 *data = (guint8 *) g_mapped_file_get_contents (mfile);
   gsize len = g_mapped_file_get_length (mfile);
 
   if (!validate_header_basic (data, len, error))
     return NULL;
 
-  meta = g_slice_new0 (GTypelib);
+  meta = g_slice_new0 (GITypelib);
   meta->mfile = mfile;
   meta->owns_memory = FALSE;
   meta->data = data; 
@@ -2159,12 +2271,12 @@ g_typelib_new_from_mapped_file (GMappedFile  *mfile,
 
 /**
  * g_typelib_free:
- * @typelib: a #GTypelib
+ * @typelib: a #GITypelib
  *
- * Free a #GTypelib.
+ * Free a #GITypelib.
  **/
 void
-g_typelib_free (GTypelib *typelib)
+g_typelib_free (GITypelib *typelib)
 {
   if (typelib->mfile)
     g_mapped_file_unref (typelib->mfile);
@@ -2176,11 +2288,11 @@ g_typelib_free (GTypelib *typelib)
       g_list_foreach (typelib->modules, (GFunc) g_module_close, NULL);
       g_list_free (typelib->modules);
     }
-  g_slice_free (GTypelib, typelib);
+  g_slice_free (GITypelib, typelib);
 }
 
 const gchar *
-g_typelib_get_namespace (GTypelib *typelib)
+g_typelib_get_namespace (GITypelib *typelib)
 {
   return g_typelib_get_string (typelib, ((Header *) typelib->data)->namespace);
 }
@@ -2191,12 +2303,12 @@ g_typelib_get_namespace (GTypelib *typelib)
  * @symbol_name: name of symbol to be loaded
  * @symbol: returns a pointer to the symbol value
  *
- * Loads a symbol from #GTypelib.
+ * Loads a symbol from #GITypelib.
  *
  * Return value: #TRUE on success
  **/
 gboolean
-g_typelib_symbol (GTypelib *typelib, const char *symbol_name, gpointer *symbol)
+g_typelib_symbol (GITypelib *typelib, const char *symbol_name, gpointer *symbol)
 {
   GList *l;
 
