@@ -30,13 +30,13 @@ gssize regress_test_ssize (gssize in);
 gsize regress_test_size (gsize in);
 gfloat regress_test_float (gfloat in);
 gdouble regress_test_double (gdouble in);
+gunichar regress_test_unichar (gunichar in);
 time_t regress_test_timet (time_t in);
 GType regress_test_gtype (GType in);
 
 /* utf8 */
 G_CONST_RETURN char *regress_test_utf8_const_return (void);
 char *regress_test_utf8_nonconst_return (void);
-void regress_test_utf8_nonconst_in (char *in);
 void regress_test_utf8_const_in (const char *in);
 void regress_test_utf8_out (char **out);
 void regress_test_utf8_inout (char **inout);
@@ -72,9 +72,6 @@ void regress_test_array_fixed_size_int_out (int **ints);
 int *regress_test_array_fixed_size_int_return (void);
 
 /* transfer tests */
-int regress_test_array_int_in_take (int n_ints, int *ints);
-gboolean regress_test_strv_in_container (char **arr);
-
 int *regress_test_array_int_full_out(int *len);
 int *regress_test_array_int_none_out(int *len);
 void regress_test_array_int_null_in (int *arr, int len);
@@ -88,11 +85,8 @@ GList *regress_test_glist_container_return (void);
 GList *regress_test_glist_everything_return (void);
 void regress_test_glist_nothing_in (const GList *in);
 void regress_test_glist_nothing_in2 (GList *in);
-void regress_test_glist_container_in (GList *in);
-void regress_test_glist_everything_in (GList *in);
 void regress_test_glist_null_in(GSList *in);
 void regress_test_glist_null_out(GSList **out_list);
-void regress_test_glist_free (GList *in);
 
 /* GSList */
 G_CONST_RETURN GSList *regress_test_gslist_nothing_return (void);
@@ -101,11 +95,8 @@ GSList *regress_test_gslist_container_return (void);
 GSList *regress_test_gslist_everything_return (void);
 void regress_test_gslist_nothing_in (const GSList *in);
 void regress_test_gslist_nothing_in2 (GSList *in);
-void regress_test_gslist_container_in (GSList *in);
-void regress_test_gslist_everything_in (GSList *in);
 void regress_test_gslist_null_in(GSList *in);
 void regress_test_gslist_null_out(GSList **out_list);
-void regress_test_gslist_free (GSList *in);
 
 /* GHashTable */
 G_CONST_RETURN GHashTable *regress_test_ghash_null_return (void);
@@ -117,9 +108,6 @@ void regress_test_ghash_null_in (const GHashTable *in);
 void regress_test_ghash_null_out (const GHashTable **out);
 void regress_test_ghash_nothing_in (const GHashTable *in);
 void regress_test_ghash_nothing_in2 (GHashTable *in);
-void regress_test_ghash_container_in (GHashTable *in);
-void regress_test_ghash_everything_in (GHashTable *in);
-void regress_test_ghash_free (GHashTable *in);
 GHashTable *regress_test_ghash_nested_everything_return (void);
 GHashTable *regress_test_ghash_nested_everything_return2 (void);
 
@@ -149,9 +137,15 @@ typedef enum
 {
   REGRESS_TEST_VALUE1,
   REGRESS_TEST_VALUE2,
-  REGRESS_TEST_VALUE3 = 42
+  REGRESS_TEST_VALUE3 = -1
 } RegressTestEnum;
 
+typedef enum
+{
+  REGRESS_TEST_UNSIGNED_VALUE1 = 1,
+  REGRESS_TEST_UNSIGNED_VALUE2 = 0x80000000
+} RegressTestEnumUnsigned;
+
 typedef enum
 {
   REGRESS_TEST_FLAG1 = 1 << 0,
@@ -161,10 +155,15 @@ typedef enum
 
 GType regress_test_enum_get_type (void) G_GNUC_CONST;
 #define REGRESS_TEST_TYPE_ENUM (regress_test_enum_get_type ())
+GType regress_test_enum_unsigned_get_type (void) G_GNUC_CONST;
+#define REGRESS_TEST_TYPE_ENUM_UNSIGNED (regress_test_enum_unsigned_get_type ())
 GType regress_test_flags_get_type (void) G_GNUC_CONST;
 #define REGRESS_TEST_TYPE_FLAGS (regress_test_flags_get_type ())
 
 const gchar * regress_test_enum_param(RegressTestEnum e);
+const gchar * regress_test_unsigned_enum_param(RegressTestEnumUnsigned e);
+
+void regress_global_get_flags_out (RegressTestFlags *v);
 
 /* constants */
 
@@ -413,6 +412,13 @@ regress_test_fundamental_sub_object_new (const char *data);
 typedef void (*RegressTestSimpleCallback) (void);
 typedef int (*RegressTestCallback) (void);
 typedef int (*RegressTestCallbackUserData) (gpointer user_data);
+/**
+ * RegressTestCallbackFull:
+ * @foo: the investment rate
+ * @bar: how much money
+ * @path: (type filename): Path to file
+ */
+typedef int (*RegressTestCallbackFull) (int foo, double bar, char *path);
 
 void regress_test_simple_callback (RegressTestSimpleCallback callback);
 int regress_test_callback (RegressTestCallback callback);
@@ -539,4 +545,58 @@ void regress_random_function_with_skipped_structure (int x,
 						     RegressSkippedStructure *foo,
 						     double v);
 
+/**
+ * RegressIntset:
+ *
+ * Like telepathy-glib's TpIntset.
+ *
+ */
+typedef struct _RegressIntset RegressIntset;
+
+/**
+ * RegressIntSet: (skip)
+ *
+ * Compatibility typedef, like telepathy-glib's TpIntSet
+ *
+ */
+typedef RegressIntset RegressIntSet;
+
+/**
+ * RegressPtrArrayAlias:
+ *
+ * Typedef'd GPtrArray for some reason
+ */
+typedef GPtrArray RegressPtrArrayAlias;
+
+void regress_introspectable_via_alias (RegressPtrArrayAlias *data);
+
+/**
+ * RegressVaListAlias:
+ *
+ * Typedef'd va_list for additional reasons
+ */
+typedef va_list RegressVaListAlias;
+
+void regress_not_introspectable_via_alias (RegressVaListAlias ok);
+
+/* private testing */
+
+typedef struct {
+  gint this_is_public_before;
+  /* < private > */
+  gint this_is_private;
+  /* < public > */
+  gint this_is_public_after;
+} RegressTestPrivateStruct;
+
+typedef enum {
+  REGRESS_TEST_PUBLIC_ENUM_BEFORE = 1 << 0,
+  /* <private> */
+  REGRESS_TEST_PRIVATE_ENUM       = 1 << 1,
+  /* <public> */
+  REGRESS_TEST_PUBLIC_ENUM_AFTER  = 1 << 2,
+} RegressTestPrivateEnum;
+
+#define REGRESS_UTF8_CONSTANT "const \xe2\x99\xa5 utf8"
+
 #endif /* __GITESTTYPES_H__ */
